﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;
using Reaktion;

public class Controller : MonoBehaviour
{
    public List<Note> attachedNotes = new List<Note>();
    
    [Header("Physics")]
    AreaEffector2D effector;
    [AutoFind(typeof(Rigidbody2D))] public Rigidbody2D rb;
    [AutoFind(typeof(SpriteRenderer),true)] public SpriteRenderer visuals;
    float angleOffset;
    public float invincibilityTime;

    private void Start()
    {
        GameObject effectorGameObject = new GameObject("Effector");
        CircleCollider2D effectorCollider = effectorGameObject.AddComponent<CircleCollider2D>();
        effectorCollider.radius = 8f;
        effectorCollider.isTrigger = true;
        effectorCollider.usedByEffector = true;
        effector = effectorGameObject.AddComponent<AreaEffector2D>();
        effector.useGlobalAngle = true;
    }

    public void AddNote(Note _note, Vector3 _position)
    {
        attachedNotes.Add(_note);

        GameObject _newAnchor = new GameObject("Anchor");
        _newAnchor.transform.parent = transform;
        _newAnchor.transform.localPosition = Vector3.zero;
        GameObject _newNote = new GameObject("Note");
        _newNote.transform.parent = _newAnchor.transform;
        _newNote.transform.localPosition = new Vector3(0f, 0.8f, 0f);
        Rigidbody2D _newNoteRigidbody = _newNote.AddComponent<Rigidbody2D>();
        _newNoteRigidbody.bodyType = RigidbodyType2D.Kinematic;
        _newNoteRigidbody.constraints = RigidbodyConstraints2D.FreezeRotation;

        _note.anchor = _newAnchor.transform;
        _note.gameObject = _newNote;

        GameObject _newNoteVisuals = Instantiate(GameManager.instance.notePrefab, _position, Quaternion.identity);
        NoteComponent _newNoteComponent = _newNoteVisuals.GetComponent<NoteComponent>();
        TargetJoint2D _newNoteJoint = _newNoteComponent.joint;
        DOTween.To(() => 0f, x => _newNoteJoint.frequency = x, GameManager.instance.noteSpringFrequency, 2f);
        DOTween.To(() => 1f, x => _newNoteJoint.dampingRatio = x, GameManager.instance.noteSpringDamping, 2f);

        _note.visuals = _newNoteVisuals;
        _note.joint = _newNoteJoint;
        _newNoteComponent.note = _note;
        DOTween.To(() => 0f, x => _note.audioSource.volume = x, _note.volume, .5f);

        Reaktor _reaktor = _note.visuals.GetComponentInChildren<Reaktor>();
        TransformGear _gear = _note.visuals.GetComponentInChildren<TransformGear>();

        _reaktor.injector.mode = GenericLink<InjectorBase>.Mode.ByReference;
        _reaktor.injector.reference = _note.injector;
    }

    public void RemoveNote(Note _note)
    {
        if (_note == null) return;

        attachedNotes.Remove(attachedNotes.Find(x => x == _note));
        DOTween.To(() => _note.audioSource.volume, x => _note.audioSource.volume = x, 0f, .5f);
        Destroy(_note.anchor.gameObject);
        Destroy(_note.visuals);
    }

    private void OnDrawGizmos()
    {
        foreach(Note _note in attachedNotes)
        {
            if (_note.gameObject != null) Gizmos.DrawSphere(_note.gameObject.transform.position, .1f);
        }
    }

    private float dampedAlpha;
    private float refVelAlpha;
    private void Update()
    {
        invincibilityTime -= Time.deltaTime;
        invincibilityTime = Mathf.Clamp(invincibilityTime, 0f, GameManager.instance.invincibilityPeriod);

        if (invincibilityTime > 0)
        {
            visuals.color = new Color(255f, 255f, 255f, (Mathf.Sin(Time.time * 10f) + 1.5f) * .5f);
        }
        else
        {
            dampedAlpha = Mathf.SmoothDamp(visuals.color.a, 1f, ref refVelAlpha, .5f);
            visuals.color = new Color(255f, 255f, 255f, dampedAlpha);
        }

        effector.transform.position = this.transform.position;

        if (Input.GetKeyDown(KeyCode.F1)) AddNote(GameManager.instance.notes[attachedNotes.Count],transform.position);
        if (Input.GetKeyDown(KeyCode.F2)) RemoveNote(attachedNotes[0]);
    }

    private Direction lastDirection;
    private float desiredAngle;
    private float refVel;
    private float refVelVolume;
    private void FixedUpdate()
    {
        effector.forceMagnitude = GameManager.instance.windForce;
        if (Input.GetKey(KeyCode.RightArrow) && Input.GetKey(KeyCode.LeftArrow))
        {
            effector.forceMagnitude = 0f;
            desiredAngle = 0f;
            GameManager.instance.windSource.volume = Mathf.SmoothDamp(GameManager.instance.windSource.volume, .2f, ref refVelVolume, .5f);
            /*if (lastDirection == Direction.right) angleOffset -= 1f;
            if (lastDirection == Direction.left) angleOffset += 1f;*/
        }
        else if (Input.GetKey(GameManager.instance.rightKey))
        {
            effector.forceAngle = GameManager.instance.windAngle;
            desiredAngle = -10f;
            /*angleOffset -= 1f;
            lastDirection = Direction.right;*/
            GameManager.instance.windSource.volume = Mathf.SmoothDamp(GameManager.instance.windSource.volume, .4f, ref refVelVolume, .5f);
        }
        else if (Input.GetKey(GameManager.instance.leftKey))
        {
            effector.forceAngle = 180 - GameManager.instance.windAngle;
            desiredAngle = 10f;
            /*angleOffset += 1f;
            lastDirection = Direction.left;*/
            GameManager.instance.windSource.volume = Mathf.SmoothDamp(GameManager.instance.windSource.volume, .4f, ref refVelVolume, .5f);
        }
        else
        {
            effector.forceMagnitude = 0f;
            desiredAngle = 0f;
            GameManager.instance.windSource.volume = Mathf.SmoothDamp(GameManager.instance.windSource.volume, .2f, ref refVelVolume, .5f);
            /*if (lastDirection == Direction.right) angleOffset -= 1f;
            if (lastDirection == Direction.left) angleOffset += 1f;*/
        }

        angleOffset -= 2f;
        visuals.transform.eulerAngles = new Vector3(0f,0f,Mathf.SmoothDampAngle(visuals.transform.eulerAngles.z, desiredAngle, ref refVel, 0.2f));

        for (int i = 0; i < attachedNotes.Count; i++)
        {
            if (attachedNotes[i].anchor != null)
            {
                attachedNotes[i].anchor.eulerAngles = new Vector3(0f, 0f, i * (360 / attachedNotes.Count) + angleOffset);
            }
        }

        foreach (Note _note in attachedNotes)
        {
            _note.joint.target = _note.gameObject.transform.position;
        }
    }

    private Vector2 averageNormal;
    private ContactPoint2D[] contacts = new ContactPoint2D[10];
    private void OnCollisionEnter2D(Collision2D collision)
    {
        collision.GetContacts(contacts);
        foreach (ContactPoint2D _contact in contacts)
        {
            averageNormal += _contact.normal;
        }
        averageNormal /= contacts.Length;
        
        rb.AddForce(averageNormal.normalized * GameManager.instance.repulseForce,ForceMode2D.Impulse);

        if (invincibilityTime > 0)
        {
            return;
        }
        if (attachedNotes.Count != 0) RemoveNote(attachedNotes[0]);
        invincibilityTime = GameManager.instance.invincibilityPeriod;
        GameManager.instance.hurtSource.Play();
    }
}

public enum Direction
{
    left,
    right
}
