﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Sirenix.OdinInspector;
using DG.Tweening;
using Reaktion;

public class GameManager : MonoBehaviour {

    public Animator bardAnim;

    public SpriteRenderer credits;
    public AudioClip hurtClip;
    public AudioClip windClip;
    public Transform heartSpawnPoint;
    public AudioInjector injector;
    public AudioSource heartBeatSource;
    [Title("Tweaking")]
    public int limiteDecue;
    public int limiteNeutre;
    public int limiteContente;
    public float invincibilityPeriod = 5f;
    public float noteSpringDamping;
    public float noteSpringFrequency;
    public float repulseForce;
    [Title("Wind")]
    public float windForce;
    public float windAngle;
    [Title("Keybindings")]
    public KeyCode rightKey = KeyCode.Escape;
    public KeyCode leftKey = KeyCode.Escape;

    public List<Note> notes = new List<Note>();
    
    [HideInInspector] public GameObject notePrefab;
    [HideInInspector] public Controller controller;
    public static GameManager instance;
    public StartScreenReferencer startScreenReferencer;
    public CameraBehavior cameraBehavior;

    public AudioSource windSource;
    public AudioSource hurtSource;

	void Awake ()
    {
        InitInstance();

        startScreenReferencer = FindObjectOfType<StartScreenReferencer>();
        cameraBehavior = FindObjectOfType<CameraBehavior>();

        controller = FindObjectOfType<Controller>();
        notePrefab = Resources.Load("Prefabs/notePrefab") as GameObject;
        
        foreach(Note _note in notes)
        {
            GameObject _audioGameObject = new GameObject("note sound");
            _audioGameObject.transform.parent = transform;

            _note.audioSource = _audioGameObject.AddComponent<AudioSource>();
            _note.injector = _audioGameObject.AddComponent<AudioInjector>();
            _note.injector.mute = false;
            _note.audioSource.clip = _note.sound;
            _note.audioSource.loop = true;
            _note.audioSource.volume = 0f;
            _note.audioSource.Play();
        }
        GetComponentInChildren<AudioSource>().Play();
	}

    private void Start()
    {
        bardAnim.SetBool("isIdling", true);
        injector = GetComponentInChildren<AudioInjector>();

        StartCoroutine(GameIntro());
        cameraBehavior.mode = CameraBehavior.Mode.position;

        windSource = gameObject.AddComponent<AudioSource>();
        windSource.clip = windClip;
        windSource.volume = .3f;
        windSource.loop = true;
        windSource.Play();

        hurtSource = gameObject.AddComponent<AudioSource>();
        hurtSource.clip = hurtClip;
        hurtSource.volume = .8f;
        hurtSource.playOnAwake = false;
    }

    public KeyCode currentKey;
    bool keyIsPressed = false;
    bool askingForKey = false;
    private void Update()
    {
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            Application.Quit();
        }

        currentKey = FetchKey();

        if (Input.anyKeyDown && askingForKey)
        {
            if (Input.GetMouseButtonDown(0) || Input.GetMouseButtonDown(1) || Input.GetMouseButtonDown(2)) return;
            keyIsPressed = true;
        }

        if (Input.GetKeyDown(KeyCode.Insert))
        {
            FadeToBlack();
        }
    }

    bool keysAreSet;
    float screenIn = 3f;
    float screenOut = 3f;
    float screenInFast = 1f;
    float screenOutFast = 1f;
    float zoomOut = 6f;
    SpriteRenderer blackSprite;
    IEnumerator GameIntro()
    {
        GameObject _blackGameObject = new GameObject("Black");
        _blackGameObject.transform.position = cameraBehavior.transform.position + (cameraBehavior.transform.forward * 1f);
        _blackGameObject.transform.localScale = Vector3.one * 1000f;
        blackSprite = _blackGameObject.AddComponent<SpriteRenderer>();
        blackSprite.sprite = Resources.Load<Sprite>("Square");
        blackSprite.color = Color.black;
        blackSprite.sortingLayerName = "UI";

        blackSprite.DOFade(0f, 5f).SetEase(Ease.InOutCubic);
        yield return new WaitForSeconds(2f);

        if (!keysAreSet)
        {
            startScreenReferencer.instructionsText.text = "Set your right key";
            DOTween.To(() => startScreenReferencer.instructionsText.alpha, x => startScreenReferencer.instructionsText.alpha = x, 1f, screenIn).SetEase(Ease.InOutCubic);
            yield return new WaitForSeconds(screenIn - 2);

            askingForKey = true;
            while (true)
            {
                if (keyIsPressed)
                {
                    keyIsPressed = false;
                    rightKey = currentKey;
                    StartCoroutine(KeyPrompt(rightKey.ToString()));
                    break;
                }
                yield return new WaitForEndOfFrame();
            }
            askingForKey = false;

            DOTween.To(() => startScreenReferencer.instructionsText.alpha, x => startScreenReferencer.instructionsText.alpha = x, 0f, screenOutFast).SetEase(Ease.InOutCubic);
            yield return new WaitForSeconds(screenOutFast);
            startScreenReferencer.instructionsText.text = "Set your left key";
            DOTween.To(() => startScreenReferencer.instructionsText.alpha, x => startScreenReferencer.instructionsText.alpha = x, 1f, screenInFast).SetEase(Ease.InOutCubic);
            yield return new WaitForSeconds(screenInFast - 1);

            askingForKey = true;
            while (true)
            {
                if (keyIsPressed)
                {
                    keyIsPressed = false;
                    if (currentKey != rightKey)
                    {
                        leftKey = currentKey;
                        StartCoroutine(KeyPrompt(leftKey.ToString()));
                        break;
                    }
                }
                yield return new WaitForEndOfFrame();
            }
            askingForKey = false;

            DOTween.To(() => startScreenReferencer.instructionsText.alpha, x => startScreenReferencer.instructionsText.alpha = x, 0f, screenOutFast).SetEase(Ease.InOutCubic);
            yield return new WaitForSeconds(screenOutFast);
        }
        keysAreSet = true;

        DOTween.To(() => cameraBehavior.desiredPosition, x => cameraBehavior.desiredPosition = x, new Vector3(cameraBehavior.desiredPosition.x, cameraBehavior.desiredPosition.y,-23f), zoomOut).SetEase(Ease.InOutCubic);
        yield return new WaitForSeconds(zoomOut);
        bardAnim.SetTrigger("throw");
        startScreenReferencer.titleSpriteRenderer.DOFade(0f, 5f).SetEase(Ease.InOutCubic);
        yield return new WaitForSeconds(1f);
        BeginGame();
    }

    public void FadeToBlack()
    {
        blackSprite.transform.position = cameraBehavior.transform.position + (cameraBehavior.transform.forward * 1f);
        blackSprite.DOFade(1f, 5f).SetEase(Ease.InOutCubic);
        StartCoroutine(Credits());
    }

    public IEnumerator Credits()
    {
        yield return new WaitForSeconds(5f);
        credits.DOFade(1f, 3f).SetEase(Ease.InOutCubic);
    }

    void BeginGame()
    {
        heartBeatSource.volume = 1f;
        controller = (Instantiate(Resources.Load("Prefabs/Heart"), heartSpawnPoint.position, Quaternion.identity) as GameObject).GetComponent<Controller>();
        controller.rb.AddForce((Vector2.up*5f + Vector2.right*5f),ForceMode2D.Impulse);
        controller.GetComponentInChildren<Reaktor>().injector.mode = GenericLink<InjectorBase>.Mode.ByReference;
        controller.GetComponentInChildren<Reaktor>().injector.reference = injector;

        cameraBehavior.mode = CameraBehavior.Mode.target;
        cameraBehavior.targets.Add(controller.transform);
    }

    IEnumerator KeyPrompt(string _str)
    {
        startScreenReferencer.keyConfirmation.text = _str;
        DOTween.To(() => startScreenReferencer.keyConfirmation.alpha, x => startScreenReferencer.keyConfirmation.alpha = x, 1f, screenInFast).SetEase(Ease.InOutCubic);
        yield return new WaitForSeconds(screenInFast);
        DOTween.To(() => startScreenReferencer.keyConfirmation.alpha, x => startScreenReferencer.keyConfirmation.alpha = x, 0f, screenOutFast).SetEase(Ease.InOutCubic);
        yield break;
    }

    int e;
    KeyCode FetchKey()
    {
        e = System.Enum.GetNames(typeof(KeyCode)).Length;
        for (int i = 0; i < e; i++)
        {
            if (Input.GetKey((KeyCode)i))
            {
                return (KeyCode)i;
            }
        }

        return KeyCode.None;
    }

    void InitInstance()
    {
        if (instance != null)
        {
            Destroy(gameObject);
        }
        else
        {
            instance = this;
            DontDestroyOnLoad(gameObject);
            DOTween.Init();
        }
    }
}
